#!/bin/perl

#
# allegro_npl.pl :  Prepare Allegro output for displaying with gnuplot
# Syntax         :  perl allegro_npl.pl [-<option>] [<infile>]
# Infiles        :  <Argument1> (default)
# Outfiles       :  myplot.gp, chr_line zlr.plt, npl.plt and nplod.plt
# Created        : 1-Dec-2004	FR transfered from allegro_lod.pl
# Modified       : 20-Jan-2005	FR

use Cwd;
use strict;

print "\n\n";
print " * * * * * * * * * * * ALLEGRO_NPL.PL v1.0  * * * * * * * * * * * *\n";
print " * * * * * * * * View Allegro results with GNUPLOT  * * * * * * * *\n";

my $deletes=1;
my $hline=1;
my $legend=1;
my $label=1;
my $lod=0;
my $title=1;
my $xtics=1;
my $file="";


if ($#ARGV < 0) { print "usage: perl allegro_npl.pl <options> <file_no_extension>\n";exit 0;}

for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] eq "-h" ) { &Help;}
	elsif ($ARGV[$i] eq "-v" ) { &Help;}
	elsif ($ARGV[$i] eq "-nd" ) { $deletes=0;}
	elsif ($ARGV[$i] =~ "-clean" ) { &Clean;}
	elsif ($ARGV[$i] eq "-noxtics" ) {$xtics=0;}
	elsif ($ARGV[$i] =~ "-nohline" ) {$hline=0;}
	elsif ($ARGV[$i] eq "-nt" ) {$title=0;}
	elsif ($ARGV[$i] eq "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] eq "-nlabel" ) {$label=0;}
	elsif ($ARGV[$i] eq "-nplzlr" ) {$lod=4;}
	elsif ($ARGV[$i] eq "-npl" ) {$lod=3;}
	elsif ($ARGV[$i] eq "-zlr" ) {$lod=2;}
	elsif ($ARGV[$i] eq "-lod" ) {$lod=1;}
	else {$file=$ARGV[$i]; print "Argument: $ARGV[$i]\n";}
}

print "\nRead ...\n";

my @xsums=();
my $xsum = 0.0;
my $lastxsum=0.;
my $xmin = 0;

my $ymax = 2;
my $ylodmax=2;
my $ymin = -1;

my $pfad = getcwd();
my $datum = scalar localtime;
my %chrlabel=();
my $status=0;
my $in_file="";
my $chr="";

#######################################    M A I N   #############################															
open (out1,">$pfad/nplod.plt") || die "could not open nplod.plt :$!\n";
open (out2,">$pfad/npl.plt") || die "could not open npl.plt :$!\n";
open (out3,">$pfad/zlr.plt") || die "could not open zlr.plt :$!\n";

my @dirs= qw(c01 c02 c03 c04 c05 c06 c07 c08 c09 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20 c21 c22 c23 c24); # or cx,cxy,c25
# my @dirs= qw(c19);
foreach my $dir (@dirs) {	
	if (-d $dir) {
		$status=0;
		$chr=$dir;
		$chr=~ s/c//;
		$in_file = "${dir}/${file}\.${chr}";	# no sets of markers, one outfile per chromosome
#		unlink glob("ps001_mpt.*");
#		unlink glob("ps100_mpt.*");
		if (-f "$in_file") {	 
			&ReadNPL();
		}
		else { 
			my @set=glob("${dir}/${file}_*.*"); # moving window with sets of markers
			my $nr=$#set+1;
			foreach my $i ( 1..$nr) {		
				$in_file = "${dir}/${file}_${i}\.${chr}";
				if (-f "$in_file") {	  
					&ReadNPL(); 
				}
				else { print "no file: $in_file in $dir\n";}
			}
		} 
		if ($status == 1) {
			my $xpos=(($lastxsum + $xsum)/2);
			my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
			if ($var > 9){ $xpos -=30;}
			else { $xpos -=5;}
			if ($var eq "23") {$var = "X";}
			if ($var == "24") {$var = "XY";}
			if ($var == "25") {$var = "XY";}
			$chrlabel{$var}=$xpos;
			$lastxsum=$xsum;
		}
		push (@xsums,$xsum);
	}
	else { print "$dir not existing \n";
	}
} # end foreach

close (out1);
close (out2);
close (out3);
&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot myplot.gp\" \n\n";
system ("gnuplot.exe myplot.gp");

if ($deletes == 1) { &Clean;}

#################################   R E A D L O D   #################################  

sub ReadNPL {
	my @v=();
	my $xmax = 0.0;
	open (IN, "$in_file") || die "could not open $in_file :$!\n";
	print "File: $in_file\n"; 
	$status =1;
	while (<IN>){
		$_ =~ s/-Inf/-99\.99/g;
		$_ =~ s/-inf/-99\.99/g;
		$_ =~ s/nan/-1\.50/g;
		chomp;
		@v = split(' ',$_);
		
		if ($. > 1) {
		    if ($v[5] eq  "_") {next;}
			if ($v[2] < 0.) { $v[1] = -$v[1];} 
			printf(out1 "%f %s\n", ($v[0] + $xsum), $v[1]); # npLOD
			printf(out2 "%f %s\n", ($v[0] + $xsum), $v[3]); # NPL
			printf(out3 "%f %s\n", ($v[0] + $xsum), $v[4]); # Zlr
			if ($v[0] > $xmax) { $xmax = $v[0];}  
			if ($v[1] > $ylodmax) { $ylodmax = $v[1];} # dynamische Anpassung der y-Achse
			if ($v[3] > $ymax) { $ymax = $v[3];} #
			if ($v[4] > $ymax) { $ymax = $v[4];} #
		}
	} # end foreach
	close (IN);
	$xsum = $xsum + $xmax;
} ## end ReadNPL

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {
	
#   $ymax=4.0;
#	$xsum = 3700;		# for a fixed axle in every picture
	if ($lod == 1) { $ymax =$ylodmax;}
	$ymax += 1;						# y-achse auf ganze LOD-Zahl bringen
	my @buff =	split('\.',$ymax);		# round up to integer
	$ymax=$buff[0];
#	$ymin= -$ymax;
	
	open (OUT,"> chr_line") || die "could not open chr_line :$!\n";

	if ($hline=1){
		for (my $i=$ymin+1; $i<$ymax;$i++) {	# horizontale Linien bei LODs = 1,2,3,4,..
			print OUT "0 $i\n$xsum $i\n\n";
		}
	}

	for (my $k=0;$k < $#xsums;$k++){			# vertikale Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT "$xsums[$k] $ymin \n$xsums[$k] $ymax \n\n";         
	}
	close (OUT);
}
###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	
	my $titel = "$pfad - Allegro $file   - $datum";
	
	open (out5,"> myplot.gp") || die "could not open myplot.gp :$!\n";

	print out5 "set autoscale\n";
	if ($title == 1) {print out5 "set title \"$titel\"\n";}
	print out5 "set xlabel \"cM\"\n";

	if ($label == 1) {
		my $ypos = $ymax + (($ymax-$ymin)/40);
		foreach (keys %chrlabel) {print out5 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	}
	print out5 "set ytics $ymin,1,$ymax\n";
	if ($xtics == 0) {print out5 "set noxtics\n";}
	print out5 "set size 1,0.95\n";
	if ($legend == 0){ print out5 "set nokey\n";}
	print out5 "set terminal postscript color solid \"Helvetica\" \n";

	if ($lod == 1) { 
		print out5 "set ylabel \"LOD\*\"\n"; 
		print out5 "set output \"allegro_${file}_LOD.ps\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line\' notitle with lines lt 0, \'nplod.plt\' title 'LOD\*' with lines lt 3\n";
	}
	elsif ($lod == 2) { 
		print out5 "set ylabel \"Zlr\"\n"; 
		print out5 "set output \"allegro_${file}_Zlr.ps\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line\' notitle with lines lt 0, \'zlr.plt\' with lines lt 1\n";
	}
	elsif ($lod == 3) { 
		print out5 "set ylabel \"NPL score\"\n"; 
		print out5 "set output \"allegro_${file}_NPL.ps\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line\' notitle with lines lt 0, \'npl.plt\' with lines lt 1\n";
	}
	elsif ($lod == 4) { 
		print out5 "set ylabel \"NPL/Zlr score\"\n"; 
		print out5 "set output \"allegro_${file}_NPLZlr.ps\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line\' notitle with lines lt 0, \'npl.plt\' title 'NPL' with lines lt 1, \'zlr.plt\' title 'Zlr' with lines lt 5\n";
	}	
	else {
		print out5 "set ylabel \"NPL/Zlr/LOD\* score\"\n"; 
		print out5 "set output \"allegro_${file}_mix.ps\"\n";
		print out5 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'chr_line\' notitle with lines lt 0, \'zlr.plt\' title 'Zlr' with lines lt 1, \'npl.plt\' title 'NPL' with lines lt 2, \'nplod.plt\' title 'LOD\*' with lines lt 3\n";		
	}
	
	print out5 "set terminal x11 color \"Helvetica\" 12\n";
	print out5 "replot\n";
	print out5 "\npause -1 \"Hit return to continue\"\n\n";

	close (out5);

} ## end pr_batch
					

#################################   H E L P   #################################  
sub Help {

print " * * * * * *  Created by Franz Rueschendorf 1-Dec-2004  * * * * * * \n";
print " * * * * * * * * *  Max-Delbrueck-Center, Berlin  * * * * * * * * * \n";
print " * * * * * * * * *  email: fruesch\@mdc-berlin.de  * * * * * * * * * \n\n";
print "  Usage        : perl [path]allegro_npl.pl [<-options>] <infile_no_extension>\n";
print "  Infiles      : <npl_file>.chr \n";
print "  Outfiles     : myplot.gp, chr_line, npl.plt, zlr.plt and nplod.plt \n" ;
print "  View results : gnuplot myplot.gp\n\n";
exit 0;
}

################################   C L E A N   ################################  
#
# removes files in the current folder with the extension *.plt *.gp and chr_line
#
sub Clean { 
unlink glob("*.plt");
unlink glob("*.gp");
unlink "chr_line";
# exit 0;
}

###############################################################################
### end allegro_npl.pl
